/* 8BitAmEthernet - BlueMini
 * version 0.1.2 date 2007-12-29
 * Copyright 2005-2007 Stefan Schuermans <stefan@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a project of CCC-AC - http://www.cccac.de/
 */

#include <avr/pgmspace.h>

#include "mcuf.h"
#include "ser8.h"

//idle movie data
const unsigned char PROGMEM McufIdleMovieData[] = {
#include "mcuf_idle_movie_data.inc"
};

//idle time of MCUF (in ticks (20ms))
#define MCUF_IDLE_TICKS 500 //10s
static unsigned int McufIdleTicks = 0;

//current delay of idle movie (in ticks (20ms))
static unsigned int McufIdleMovieDelay;
//current position in idle movie
static uint16_t McufIdleMoviePtr;

//send a frame to the serial ports
static void McufSendFrame( unsigned char * pPixels )
{
  unsigned char bytes[8] = { 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80 }; // sync
  ser8send( bytes );
  unsigned char y, x, i; // pixel data
  unsigned short pos_y, pos_x, pos_i;
  for( y = 0, pos_y = 0; y < 7; y++, pos_y += 98 ) {
    for( x = 0, pos_x = pos_y; x < 13; x++, pos_x++ ) {
      for( i = 0, pos_i = pos_x; i < 8; i++, pos_i += 13 )
        if( x < 7 || i < 7 ) // last module (i == 7) has only got width 7 (instead of 13)
          bytes[i] = pPixels[pos_i];
        else
          bytes[i] = 0;
      ser8send( bytes );
    }
  }
}

//received MCUF data
void McufRecv( unsigned char * pData, unsigned short Length ) // (extern)
{
  //parse header
  if( Length < 12 )
    return;
  if( pData[0] != 0x23 || pData[1] != 0x54 || pData[2] != 0x26 || pData[3] != 0x66 ) //magic
    return;
  unsigned short height = (unsigned short)pData[4] << 8 | (unsigned short)pData[5];
  unsigned short width = (unsigned short)pData[6] << 8 | (unsigned short)pData[7];
  unsigned short channels = (unsigned short)pData[8] << 8 | (unsigned short)pData[9];
  unsigned short maxval = (unsigned short)pData[10] << 8 | (unsigned short)pData[11];

  //check length
  if( height * width * channels > Length - 12 )
    return;

  //check format 98x7-1/16
  if( height != 7 || width != 98 || channels != 1 || maxval != 15 )
    return;

  //reset idle time
  McufIdleTicks = 0;

  //send frame
  McufSendFrame( pData + 12 );
}

//restart idle movie
static void McufIdleRestart( void )
{
  McufIdleMovieDelay = 0;
  McufIdleMoviePtr = (uint16_t)McufIdleMovieData;
}

//play idle movie
static void McufIdlePlay( void )
{
  unsigned char val, pixels[7*98];
  uint16_t i;

  //delay active
  if( McufIdleMovieDelay > 0 ) {
    McufIdleMovieDelay--;
    return;
  }

  //fetch next frame
  for( i = 0; i < 7*98; i += 2 ) {
    val = pgm_read_byte_near( McufIdleMoviePtr++ );
    pixels[i+0] = val >> 4;
    pixels[i+1] = val & 0x0F;
  }
  //output next frame
  McufSendFrame( pixels );

  //fetch delay
  McufIdleMovieDelay = (unsigned int)pgm_read_byte_near( McufIdleMoviePtr++ ) << 8 |
                       (unsigned int)pgm_read_byte_near( McufIdleMoviePtr++ );

  //reset pointer if end reached
  if( McufIdleMoviePtr >= (uint16_t)McufIdleMovieData + sizeof( McufIdleMovieData ) )
    McufIdleMoviePtr = (uint16_t)McufIdleMovieData;
}


//tick procedure - call every 20ms
void McufTick20( void ) //(extern)
{
  //idle
  if( McufIdleTicks >= MCUF_IDLE_TICKS ) {
    //play idle movie
    McufIdlePlay( );
  }
  //not yet idle
  else {
    //count idle time
    McufIdleTicks++;
    //now idle
    if( McufIdleTicks >= MCUF_IDLE_TICKS ) {
      //restart idle movie
      McufIdleRestart( );
      //play idle movie
      McufIdlePlay( );
    }
  }
}

