; 8BitAmEthernet - BlueMini
; version 0.1 date 2007-12-06
; Copyright 2005-2007 Stefan Schuermans <stefan@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a project of CCC-AC - http://www.cccac.de/

#include <avr/io.h>

.section .text



; send data simultaneously on 8 serial ports (data must point to 8 bytes)
; extern void ser8send( unsigned char * data );
.global ser8send
.func ser8send



; crystal: 16.0000 MHz => 139 cycles for 1/115108s (about 115200 baud)



.macro generatebit sourcebit, destreg, destbit
	ld	R16, Z+
	bst	R16, \sourcebit
	bld	\destreg, \destbit
.endm
; 4 cycles



.macro generatebytes sourcebit
	generatebit	\sourcebit, R17, 0
	generatebit	\sourcebit, R17, 1
	generatebit	\sourcebit, R17, 2
	generatebit	\sourcebit, R17, 3
	generatebit	\sourcebit, R17, 4
	generatebit	\sourcebit, R17, 5
	generatebit	\sourcebit, R17, 6
	generatebit	\sourcebit, R17, 7
.endm
; 8 * 4 cycles = 32 cycles



.macro output
	sts	PORTF, R17
.endm
; 2 cycles



.macro gotofirstbyte
	mov	ZL, r24
	mov	ZH, r25
.endm
; 2 cycles



.macro outputbit sourcebit
	gotofirstbyte			; 2 cylces
	generatebytes	\sourcebit	; 32 cycles
	output				; 2 cycles
	rcall		wait103		; 103 cycles
.endm
; 139 cycles



.macro outputconst value
	rcall		wait33		; 33 cycles
	ldi		R17, \value	; 1 cycle
	output				; 2 cycles
	rcall		wait103		; 103 cycles
.endm
; 139 cycles



; send data simultaneously on 8 serial ports (data must point to 8 bytes)
; extern void ser8send( unsigned char * data );
ser8send:
	push	R16
	push	R17

	outputconst	0x00		; start bit (always low)
	outputbit	0		; eight data bits
	outputbit	1
	outputbit	2
	outputbit	3
	outputbit	4
	outputbit	5
	outputbit	6
	outputbit	7
	outputconst	0xFF		; stop bit (always high)

	pop	R17
	pop	R16
	ret



; wait 33 cycles (including rcall and ret)
wait33:
	ldi	R16, 8				; 1 cycle
wait33_loop:
	dec	R16				; N cycles
	brne	wait33_loop			; 2N-1 cycles
	nop					; 1 cycle
	nop					; 1 cycle
	ret					; 7 cycles (including rcall)


	
; wait 103 cycles (including rcall and ret)
wait103:
	ldi	R16, 32				; 1 cycle
wait103_loop:
	dec	R16				; N cycles
	brne	wait103_loop			; 2N-1 cycles
	ret					; 7 cycles (including rcall)



.endfunc
